/* suite.vala
 *
 * Copyright (C) 2008 Ali Sabil <ali.sabil@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using GLib;

namespace TestIt {

	public class Suite : Object {

		protected GLib.List<Case> cases = new GLib.List<Case> ();
		protected Case current_test_case = null;

		private bool break_mainloop = false;
		private uint timeout_source_id = 0;

		public uint total {
			get;
			private set;
			default = 0;
		}

		public uint succeeded {
			get;
			private set;
			default = 0;
		}

		public string name {
			get;
			protected set;
		}

		public Suite () {
		}

		protected void add_test_case(string name, CaseFunc case_func) {
			var test_case = new Case(name);
			test_case.func = case_func;
			cases.append (test_case);
		}

		public int run () {
			initialize ();

			total = 0;
			succeeded = 0;

			stdout.printf (">> Test suite: %s\n", name);

			Timer timer = new Timer();
			timer.start();

			foreach (Case test_case in cases) {
				current_test_case = test_case;
				current_test_case.total_assertions = 0;
				current_test_case.succeeded_assertions = 0;

				break_mainloop = false;
				timeout_source_id = 0;
				set_up ();
				if (current_test_case.func != null)
					current_test_case.func ();
				tear_down ();

				stdout.printf ("  > check #%02u:  checking %s ",
						total++,
						ellipsize(test_case.name));
				if (test_case.total_assertions == test_case.succeeded_assertions) {
					succeeded++;
					stdout.printf ("OK\n");
				} else {
					stdout.printf ("FAILED\n");
				}
			}
			timer.stop();
			stdout.printf ("  >>> Summary: %s\n", name);
			stdout.printf ("    %u/%u test(s) succeeded\n", succeeded, total);
			stdout.printf ("    test(s) executed in %f second(s)\n\n", timer.elapsed());

			return (int) (total - succeeded);
		}

		protected virtual void initialize () {
		}

		protected virtual void set_up () {
		}

		protected virtual void tear_down () {
		}

		protected void success (string? message=null) {
			var res = new AssertionResult (AssertionStatus.SUCCEEDED, message);

			current_test_case.total_assertions++;
			current_test_case.succeeded_assertions++;
			current_test_case.results.prepend (res);
		}

		protected void skip (string? message=null) {
			var res = new AssertionResult (AssertionStatus.SKIPPED, message);

			current_test_case.results.prepend (res);
		}

		protected void failure (string? message=null) {
			var res = new AssertionResult (AssertionStatus.FAILED, message);

			current_test_case.total_assertions++;
			current_test_case.results.prepend (res);
		}

		protected void timeout (string? message=null) {
			var res = new AssertionResult (AssertionStatus.TIMED_OUT, message);

			current_test_case.total_assertions++;
			current_test_case.results.prepend (res);
		}

		protected void assert_true (bool cond, string? message=null) {
			if (cond)
				success (message);
			else
				failure (message);
		}

		protected void assert_false (bool cond, string? message=null) {
			if (! cond)
				success (message);
			else
				failure (message);
		}

		protected void async_wait (uint timeout=0) {
			var loop = new MainLoop (null, false);
			var context = loop.get_context ();

			if (timeout > 0 && !break_mainloop) {
				Timeout.add (timeout, timeout_callback);
			}

			while (!break_mainloop) {
				context.iteration (false);
			}
			break_mainloop = false;
		}

		protected void async_break () {
			if (timeout_source_id > 0) {
				Source.remove (timeout_source_id);
			}
			break_mainloop = true;
		}

		protected void async_success (string? message=null) {
			async_break ();
			success (message);
		}

		protected void async_skip (string? message=null) {
			async_break ();
			skip (message);
		}

		protected void async_failure (string? message=null) {
			async_break ();
			failure (message);
		}

		private bool timeout_callback () {
			timeout_source_id = 0;
			async_break ();
			timeout ();
			return false;
		}

		private static string ellipsize (string s) {
			if (s == null)
				return "";
			string result;
			if (s.len() > 48)
				result = s.substring (0, 48);
			else
				result = s;
			result +=  " " + string.nfill (52 - result.len(), '.');
			return result;
		}
	}
}
