/* base-contact-source.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 * Copyright (C) 2008 Ali Sabil <ali.sabil@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Gee;

using People.Utils;


namespace People.Backend {
	/**
	 * BaseContactSource is an abstract class implementing the
	 * {@link ContactSource} interface.
	 * <p>
	 * The user of this class MUST implement {@link BaseContactSource.validate_contact_id}
	 * and {@link BaseContactSource.create_contact_object}.
	 * <p>
	 * This stub of implementation extends {@link AsyncOperationHelper} to
	 * provide methods to deal with {@link AsyncOperation} creation and
	 * completion.
	 */
	public abstract class BaseContactSource : AsyncOperationHelper,
			ContactSource {
		private ContactManager contact_manager = new ContactManager ();
		private Gee.Map<AsyncOperation<Contact>, string> pending_results =
				new Gee.HashMap<AsyncOperation<Contact>, string> ();

		/* ContactSource interface */
		public ContactSourceFlags flags {
			get;
			protected set;
			default = ContactSourceFlags.NONE;
		}

		public AsyncOperation<Contact> request_contact (string? id) {
			if (id == null && !(ContactSourceFlags.CAN_CREATE_CONTACT in flags)) {
				var error = new Error.NOT_SUPPORTED ("Contact creation not supported by this backend");
				return sync_error (error);
			}

			if (id == null) {
				return sync_result (get_contact_object (null));
			} else {
				AsyncOperationCallback contact_id_checked = (op, data) => {
					AsyncOperation<Contact> result_op = (AsyncOperation<Contact>) data;
					string id = pending_results.get (result_op);
					pending_results.remove (result_op);

					bool valid_id = (bool) op.get_result ();
					if (valid_id) {
						var contact = get_contact_object (id);
						async_operation_emit (result_op, contact);
					} else {
						var error = new Error.INVALID_ARGUMENT ("Requested a contact with an unknown ID:" + id);
						async_operation_error (result_op, error);
					}
				};

				AsyncOperation<Contact> result_op = async_operation_init ();
				AsyncOperation<bool> op = validate_contact_id (id);
				pending_results.set (result_op, id);
				op.add_callback (contact_id_checked, result_op);
				return result_op;
			}
		}
		/* end of ContactSource interface */

		protected Contact get_contact_object (string? id) {
			Contact contact = null;

			if (id != null) {
				contact = contact_manager.request_contact (id);
			}

			if (contact == null) {
				contact = create_contact_object (id);
				register_contact (contact);
			}

			return contact;
		}

		protected void register_contact (Contact contact) {
			if (contact_manager.register_contact (contact)) {
				contact.notify["id"] += contact_state_changed;
			}
		}

		/**
		 * The user of this class MUST implement this method to allow the validation
		 * and checking for the existence of the contact ID.
		 */
		protected abstract AsyncOperation<bool> validate_contact_id (string id);

		/**
		 * The user of this class MUST implement this method returning a Contact object
		 * corresponding to the contact ID. The implementation MUST NOT check for the validity
		 * of the given ID.
		 */
		protected abstract Contact create_contact_object (string? id);

		private void contact_state_changed (Contact contact, ParamSpec pspec) {
			if (contact.id == null)
				contact_removed (contact);
			else
				contact_added (contact);
		}
	}
}
